'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var JSBI = _interopDefault(require('jsbi'));

var id = 0;

function _classPrivateFieldLooseKey(name) {
  return "__private_" + id++ + "_" + name;
}

function _classPrivateFieldLooseBase(receiver, privateKey) {
  if (!Object.prototype.hasOwnProperty.call(receiver, privateKey)) {
    throw new TypeError("attempted to use private field on non-instance");
  }

  return receiver;
}

/**
 * Divide and round two big integers.
 *
 * @param {BigInt|string|number} dividend Integer to be divided + rounded
 * @param {BigInt|string|number} divisor  Divisor
 * @returns {string}
 */

function divideRoundBigInt(dividend, divisor) {
  const parsedDividend = JSBI.BigInt(String(dividend));
  const parsedDivisor = JSBI.BigInt(String(divisor));
  return JSBI.divide(JSBI.add(parsedDividend, JSBI.divide(parsedDivisor, JSBI.BigInt(2))), parsedDivisor).toString();
}

const _10 = /*#__PURE__*/JSBI.BigInt(10);
/**
 * Converts an amount. The conversion rate is expressed as the amount of the output token
 * obtained per unit of the input token.
 *
 * e.g:
 * Input token: ANT
 * Output token: ETH
 * Amount of ANT: 10
 * Conversion rate: 0.5 ETH per ANT. (1 ANT = 0.5 ETH)
 * Converted Amount = 10 * 0.50 = 5 ETH.
 *
 * @param {BigInt|string|number} amount              Amount of the input token to convert.
 * @param {BigInt|string|number} decimals            Decimals of the input token to convert.
 * @param {string|number} convertRate                Rate of conversion between the input and output token.
 * @param {BigInt|string|number} targetDecimals      Decimals for the output amount.
 * @returns {string}
 */


function convertAmount(amount, decimals, convertRate, targetDecimals) {
  const parsedAmount = JSBI.BigInt(String(amount));
  const parsedDecimals = JSBI.BigInt(String(decimals));
  const parsedTargetDecimals = JSBI.BigInt(String(targetDecimals));
  const [rateWhole = '', rateDec = ''] = String(convertRate).split('.'); // Remove any trailing zeros from the decimal part

  const parsedRateDec = rateDec.replace(/0*$/, ''); // Construct the final rate, and remove any leading zeros

  const rate = JSBI.BigInt(`${rateWhole}${parsedRateDec}`.replace(/^0*/, ''));
  const ratePrecision = JSBI.BigInt(parsedRateDec.length);
  const scaledRate = JSBI.multiply(rate, JSBI.exponentiate(_10, parsedTargetDecimals));
  const convertedAmount = divideRoundBigInt(divideRoundBigInt(JSBI.multiply(parsedAmount, scaledRate), JSBI.exponentiate(_10, ratePrecision)), JSBI.exponentiate(_10, parsedDecimals));
  return String(convertedAmount);
}

const NO_BREAK_SPACE = '\u00a0';

/**
 * Formats a number for display purposes.
 *
 * This function is not using Intl.NumberFormat() to be compatible with big
 * integers expressed as string, or BigInt-like objects.
 *
 * @param {BigInt|string|number} number Number to convert
 * @returns {string}
 */

function formatNumber(number) {
  const numAsString = String(number);
  const [integer, decimals] = numAsString.split('.');
  return [...integer].reverse().reduce((result, digit, index) => {
    return `${digit}${index > 0 && index % 3 === 0 ? ',' : ''}${result}`;
  }, decimals ? `.${decimals}` : '');
}
/**
 * Formats a token amount for display purposes.
 *
 * @param {BigInt|string|number} amount              Number to round
 * @param {BigInt|string|number} decimals            Decimal placement for amount
 * @param {BigInt|string|number} digits              Rounds the number to a given decimal place
 * @param {boolean}              options.commify     Decides if the formatted amount should include commas
 * @param {boolean}              options.displaySign Decides if the sign should be displayed
 * @param {string}               options.symbol      Symbol for the token amount
 * @returns {string}
 */

function formatTokenAmount(amount, decimals, options = {}) {
  const {
    commify = true,
    digits = 2,
    symbol = '',
    displaySign = false
  } = options;
  let parsedAmount = JSBI.BigInt(String(amount));
  const parsedDecimals = JSBI.BigInt(String(decimals));
  let parsedDigits = JSBI.BigInt(String(digits));

  const _0 = JSBI.BigInt(0);

  const _10 = JSBI.BigInt(10);

  if (JSBI.lessThan(parsedDecimals, _0)) {
    throw new Error('formatTokenAmount(): decimals cannot be negative');
  }

  if (JSBI.lessThan(parsedDigits, _0)) {
    throw new Error('formatTokenAmount(): digits cannot be negative');
  }

  if (JSBI.lessThan(parsedDecimals, parsedDigits)) {
    parsedDigits = parsedDecimals;
  }

  const negative = JSBI.lessThan(parsedAmount, _0);

  if (negative) {
    parsedAmount = JSBI.unaryMinus(parsedAmount);
  }

  const amountConverted = JSBI.equal(parsedDecimals, _0) ? parsedAmount : JSBI.BigInt(divideRoundBigInt(parsedAmount, JSBI.exponentiate(_10, JSBI.subtract(parsedDecimals, parsedDigits))));
  const leftPart = JSBI.divide(amountConverted, JSBI.exponentiate(_10, parsedDigits));
  const processedLeftPart = commify ? formatNumber(leftPart) : leftPart;
  const rightPart = String(JSBI.remainder(amountConverted, JSBI.exponentiate(_10, parsedDigits))).padStart(Number(parsedDigits), '0').replace(/0+$/, '');
  return [displaySign ? negative ? '-' : '+' : '', processedLeftPart, rightPart ? `.${rightPart}` : '', symbol ? `${NO_BREAK_SPACE}${symbol}` : ''].join('');
}

var _decimals = /*#__PURE__*/_classPrivateFieldLooseKey("decimals");

var _value = /*#__PURE__*/_classPrivateFieldLooseKey("value");

var _symbol = /*#__PURE__*/_classPrivateFieldLooseKey("symbol");

class TokenAmount {
  constructor(value, decimals, {
    symbol = ''
  } = {}) {
    Object.defineProperty(this, _decimals, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _value, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _symbol, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldLooseBase(this, _decimals)[_decimals] = JSBI.BigInt(String(decimals));
    _classPrivateFieldLooseBase(this, _value)[_value] = JSBI.BigInt(String(value));
    _classPrivateFieldLooseBase(this, _symbol)[_symbol] = symbol;
  }

  get decimals() {
    return JSBI.toNumber(_classPrivateFieldLooseBase(this, _decimals)[_decimals]);
  }

  get symbol() {
    return _classPrivateFieldLooseBase(this, _symbol)[_symbol];
  }

  get value() {
    return _classPrivateFieldLooseBase(this, _value)[_value].toString();
  }

  export() {
    const {
      decimals,
      symbol,
      value
    } = this;
    const data = {
      d: decimals,
      v: value
    };

    if (symbol) {
      data.s = symbol;
    }

    return JSON.stringify(data);
  }

  static import(json) {
    let data;

    try {
      data = JSON.parse(json);
    } catch (err) {
      throw new Error('TokenAmount.import(): couldn’t parse data.');
    }

    if (!data.d || !data.v) {
      throw new Error('TokenAmount.import(): invalid data format provided.');
    }

    return new TokenAmount(data.v, data.d, {
      symbol: data.s
    });
  }

  static convert(amount, convertRate, targetDecimals, options) {
    const convertedAmount = convertAmount(amount.value, amount.decimals, convertRate, targetDecimals);
    return new TokenAmount(convertedAmount, targetDecimals, options);
  }

  convert(rate, targetDecimals, options) {
    return TokenAmount.convert(this, rate, targetDecimals, options);
  }

  format(options) {
    return formatTokenAmount(_classPrivateFieldLooseBase(this, _value)[_value], _classPrivateFieldLooseBase(this, _decimals)[_decimals], {
      symbol: _classPrivateFieldLooseBase(this, _symbol)[_symbol],
      ...options
    });
  }

  toString(options) {
    return this.format(options);
  }

}
TokenAmount.format = formatTokenAmount;

exports.default = TokenAmount;
//# sourceMappingURL=token-amount.cjs.development.js.map
